@@ -22,19 +22,24 @@ import mlflow
 import pmdarima
 from mlflow.exceptions import MlflowException
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
-from mlflow.utils.environment import _mlflow_conda_env
 
 from databricks.automl_runtime.forecast import OFFSET_ALIAS_MAP
 from databricks.automl_runtime.forecast.model import ForecastModel, mlflow_forecast_log_model
 
 
-ARIMA_CONDA_ENV = _mlflow_conda_env(
-    additional_pip_deps=[
-        f"pmdarima=={pmdarima.__version__}",
-        f"pickle=={pickle.format_version}",
-        f"pandas=={pd.__version__}",
-    ]
-)
+ARIMA_CONDA_ENV = {
+    "channels": ["conda-forge"],
+    "dependencies": [
+        {
+            "pip": [
+                f"pmdarima=={pmdarima.__version__}",
+                f"pickle=={pickle.format_version}",
+                f"pandas=={pd.__version__}",
+            ]
+        }
+    ],
+    "name": "pmdarima_env",
+}
 
 
 class AbstractArimaModel(ForecastModel):
@@ -140,7 +145,7 @@ class ArimaModel(AbstractArimaModel):
 
     def _predict_impl(self, input_df: pd.DataFrame) -> pd.DataFrame:
         df = input_df.rename(columns={self._time_col: "ds"})
-        df["ds"] = pd.to_datetime(df["ds"])
+        df["ds"] = pd.to_datetime(df["ds"], infer_datetime_format=True)
         # Check if the time has correct frequency
         diff = (df["ds"] - self._start_ds) / pd.Timedelta(1, unit=self._frequency)
         if not diff.apply(float.is_integer).all():
